@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #  textcopy.py - a TEXTCOPY compatible Saori module for ninix
-#  Copyright (C) 2002-2011 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2013 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -12,7 +12,8 @@
 #  PURPOSE.  See the GNU General Public License for more details.
 #
 
-import gtk
+from gi.repository import Gtk
+from gi.repository import Gdk
 
 from ninix.dll import SAORI
 
@@ -24,7 +25,7 @@ class Saori(SAORI):
         self.clipboard = None
 
     def setup(self):
-        self.clipboard = gtk.Clipboard(selection='PRIMARY')
+        self.clipboard = Gtk.Clipboard.get(Gdk.SELECTION_PRIMARY)
         return 1
 
     def finalize(self):
@@ -34,11 +35,12 @@ class Saori(SAORI):
     def execute(self, argument):
         if not argument or self.clipboard is None:
             return self.RESPONSE[400]
-        text = argument[0].encode('utf-8')
-        self.clipboard.set_text(text)
+        text = argument[0]
+        self.clipboard.set_text(text, -1)
         if len(argument) >= 2 and argument[1] != 0:
-            return 'SAORI/1.0 200 OK\r\n' \
-                   'Result: %s\r\n\r\n' % \
-                   argument[0].encode(self.charset, 'replace')
+            return b''.join((b'SAORI/1.0 200 OK\r\n',
+                             b'Result: ',
+                             argument[0].encode(self.charset, 'replace'),
+                             b'\r\n\r\n'))
         else:
             return self.RESPONSE[204]
