@@ -2,7 +2,7 @@
 #
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2011 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2013 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -12,56 +12,60 @@
 #  PURPOSE.  See the GNU General Public License for more details.
 #
 
+import codecs
 import logging
 import os
 import re
 import sys
+from collections import OrderedDict
 
 import ninix.config
 import ninix.alias
 import ninix.dll
 
 
+PLUGIN_STANDARD = (2.5, 2.5)
+
 def get_ninix_home():
-    return os.path.join(os.path.expanduser('~'), '.ninix')
+    return os.path.join(os.path.expanduser(b'~'), b'.ninix')
 
 def get_archive_dir():
-    return os.path.join(get_ninix_home(), 'archive')
+    return os.path.join(get_ninix_home(), b'archive')
 
 def get_pango_fontrc():
-    return os.path.join(get_ninix_home(), 'pango_fontrc')
+    return os.path.join(get_ninix_home(), b'pango_fontrc')
 
 def get_preferences():
-    return os.path.join(get_ninix_home(), 'preferences')
+    return os.path.join(get_ninix_home(), b'preferences')
 
-def get_normalized_path(path, encode=1):
+def get_normalized_path(path):
     path = path.replace('\\', '/')
-    if encode:
-        path = path.encode('utf-8')
-    if not os.path.isabs(path): # XXX
+    if not os.path.isabs(os.fsencode(path)): # XXX
         path = path.lower()
-    return os.path.normpath(path)
+    return os.path.normpath(os.fsencode(path))
 
 def load_config():
     if not os.path.exists(get_ninix_home()):
         return None
     ghosts = search_ghosts()
     balloons = search_balloons()
+    plugins = search_plugins()
     nekoninni = search_nekoninni()
     katochan = search_katochan()
     kinoko = search_kinoko()
-    return ghosts, balloons, nekoninni, katochan, kinoko
+    return ghosts, balloons, plugins, nekoninni, katochan, kinoko
 
-def get_shiori(path):
+def get_shiori():
     table = {}
-    shiori_lib = ninix.dll.Library('shiori', path, saori_lib=None)
+    shiori_lib = ninix.dll.Library('shiori', saori_lib=None)
+    path = ninix.dll.get_path()
     for filename in os.listdir(path):
         if os.access(os.path.join(path, filename), os.R_OK):
             name = None
             basename, ext = os.path.splitext(filename)
             ext = ext.lower()
-            if ext in ['.py', '.pyc']:
-                name = basename
+            if ext in [b'.py', b'.pyc']:
+                name = os.fsdecode(basename)
             if name and name not in table:
                 shiori = shiori_lib.request(('', name))
                 if shiori:
@@ -70,19 +74,19 @@ def get_shiori(path):
 
 def search_ghosts(target=None):
     home_dir = get_ninix_home()
-    ghosts = []
+    ghosts = OrderedDict()
     if target:
         dirlist = []
         dirlist.extend(target)
     else:
         try:
-            dirlist = os.listdir(os.path.join(home_dir, 'ghost'))
+            dirlist = os.listdir(os.path.join(home_dir, b'ghost'))
         except OSError:
             dirlist = []
-    shiori_table = get_shiori(os.path.join(sys.path[0], 'ninix/dll')) # XXX
+    shiori_table = get_shiori()
     for subdir in dirlist:
-        prefix = os.path.join(home_dir, 'ghost', subdir)
-        ghost_dir = os.path.join(prefix, 'ghost', 'master')
+        prefix = os.path.join(home_dir, b'ghost', subdir)
+        ghost_dir = os.path.join(prefix, b'ghost', b'master')
         desc = read_descript_txt(ghost_dir)
         if desc is None:
             desc = ninix.config.null_config()
@@ -90,7 +94,7 @@ def search_ghosts(target=None):
         # find a pseudo AI, shells, and a built-in balloon
         candidate = {'name': '', 'score': 0}
         # SHIORI compatible modules
-        for name, shiori in shiori_table.iteritems():
+        for name, shiori in shiori_table.items():
             score = int(shiori.find(ghost_dir, shiori_dll))
             if score > candidate['score']:
                 candidate['name'] = name
@@ -102,19 +106,23 @@ def search_ghosts(target=None):
         pos = 0 if desc.get('name') == 'default' else len(ghosts)
         use_makoto = find_makoto_dll(ghost_dir)
         ## FIXME: check surface_set
-        ghosts.insert(pos, (
-            desc, ghost_dir, use_makoto, surface_set, prefix,
-            shiori_dll, shiori_name))
+        ghosts[os.fsdecode(subdir)] = (desc, ghost_dir, use_makoto,
+                                       surface_set, prefix,
+                                       shiori_dll, shiori_name)
     return ghosts
 
-def search_balloons():
+def search_balloons(target=None):
     home_dir = get_ninix_home()
-    balloon_dir = os.path.join(home_dir, 'balloon')
-    try:
-        dirlist = os.listdir(balloon_dir)
-    except OSError:
+    balloons = OrderedDict()
+    balloon_dir = os.path.join(home_dir, b'balloon')
+    if target:
         dirlist = []
-    buf = []
+        dirlist.extend(target)
+    else:
+        try:
+            dirlist = os.listdir(balloon_dir)
+        except OSError:
+            dirlist = []
     for subdir in dirlist:
         path = os.path.join(balloon_dir, subdir)
         if not os.path.isdir(path):
@@ -130,14 +138,28 @@ def search_balloons():
             continue
         else:
             balloon_info['balloon_dir'] = (subdir, ninix.config.null_config())
-        pos = 0 if desc.get('name') == 'default' else len(buf)
-        buf.insert(pos, (desc, balloon_info))
+        balloons[os.fsdecode(subdir)] = (desc, balloon_info)
+    return balloons
+
+def search_plugins():
+    home_dir = get_ninix_home()
+    buf = []
+    plugin_dir = os.path.join(home_dir, b'plugin')
+    try:
+        dirlist = os.listdir(plugin_dir)
+    except OSError:
+        dirlist = []
+    for subdir in dirlist:
+        plugin = read_plugin_txt(os.path.join(plugin_dir, subdir))
+        if plugin is None:
+            continue
+        buf.append(plugin)
     return buf
 
 def search_nekoninni():
     home_dir = get_ninix_home()
     buf = []
-    skin_dir = os.path.join(home_dir, 'nekodorif/skin')
+    skin_dir = os.path.join(home_dir, b'nekodorif/skin')
     try:
         dirlist = os.listdir(skin_dir)
     except OSError:
@@ -152,7 +174,7 @@ def search_nekoninni():
 def search_katochan():
     home_dir = get_ninix_home()
     buf = []
-    katochan_dir = os.path.join(home_dir, 'nekodorif/katochan')
+    katochan_dir = os.path.join(home_dir, b'nekodorif/katochan')
     try:
         dirlist = os.listdir(katochan_dir)
     except OSError:
@@ -167,7 +189,7 @@ def search_katochan():
 def search_kinoko():
     home_dir = get_ninix_home()
     buf = []
-    kinoko_dir = os.path.join(home_dir, 'kinoko')
+    kinoko_dir = os.path.join(home_dir, b'kinoko')
     try:
         dirlist = os.listdir(kinoko_dir)
     except OSError:
@@ -180,7 +202,7 @@ def search_kinoko():
     return buf
 
 def read_kinoko_ini(top_dir):
-    path = os.path.join(top_dir, 'kinoko.ini')
+    path = os.path.join(top_dir, b'kinoko.ini')
     kinoko = {}
     kinoko['base'] = 'surface0.png'
     kinoko['animation'] = None
@@ -196,7 +218,7 @@ def read_kinoko_ini(top_dir):
     kinoko['extractpath'] = None
     kinoko['nayuki'] = None
     if os.access(path, os.R_OK):
-        with open(path) as f:
+        with open(path, encoding='CP932') as f:
             line = f.readline()
             if not line.strip() or line.strip() != '[KINOKO]':
                 return None
@@ -209,11 +231,11 @@ def read_kinoko_ini(top_dir):
                 if not line.strip():
                     continue
                 if '=' not in line:
-                    error = 'line %d: syntax error' % lineno
+                    error = 'line {0:d}: syntax error'.format(lineno)
                     break
                 name, value = [x.strip() for x in line.split('=', 1)]
                 if name in ['title', 'ghost', 'category']:
-                    kinoko[name] = unicode(value, 'Shift_JIS', 'ignore')
+                    kinoko[name] = value
                 elif name in ['offsetx', 'offsety']:
                     kinoko[name] = int(value)
                 elif name in ['base', 'animation', 'extractpath']:
@@ -221,29 +243,29 @@ def read_kinoko_ini(top_dir):
                 elif name in ['ontop', 'baseposition', 'baseadjust']:
                     kinoko[name] = int(value)
         if error:
-            logging.error('Error: %s\n%s (skipped)' % (error, path))
+            logging.error('Error: {0}\n{1} (skipped)'.format(error, path))
             return None
     return kinoko if kinoko['title'] else None
 
 def read_profile_txt(top_dir):
-    path = os.path.join(top_dir, 'profile.txt')
+    path = os.path.join(top_dir, b'profile.txt')
     name = None
     if os.access(path, os.R_OK):
-        with open(path) as f:
+        with open(path, 'rb') as f:
             line = f.readline()
         if line:
-            name = unicode(line.strip(), 'Shift_JIS', 'ignore')
+            name = str(line.strip(), 'CP932', 'ignore')
     if name:
         return (name, top_dir) ## FIXME
     else:
         return None
 
 def read_katochan_txt(top_dir):
-    path = os.path.join(top_dir, 'katochan.txt')
+    path = os.path.join(top_dir, b'katochan.txt')
     katochan = {}
     katochan['dir'] = top_dir
     if os.access(path, os.R_OK):
-        with open(path) as f:
+        with open(path, encoding='CP932') as f:
             name = None
             lineno = 0
             error = None
@@ -255,19 +277,19 @@ def read_katochan_txt(top_dir):
                     name = line[1:].strip()
                     continue
                 elif not name:
-                    error = 'line %d: syntax error' % lineno
+                    error = 'line {0:d}: syntax error'.format(lineno)
                     break
                 else:
                     value = line.strip()
                     if name in ['name', 'category']:
-                        katochan[name] = unicode(value, 'Shift_JIS', 'ignore')
+                        katochan[name] = value
                     if name.startswith('before.script') or \
                             name.startswith('hit.script') or \
                             name.startswith('after.script') or \
                             name.startswith('end.script') or \
                             name.startswith('dodge.script'):
                         ## FIXME: should be array
-                        katochan[name] = unicode(value, 'Shift_JIS', 'ignore')
+                        katochan[name] = value
                     elif name in ['before.fall.speed',
                                   'before.slide.magnitude',
                                   'before.slide.sinwave.degspeed',
@@ -291,54 +313,57 @@ def read_katochan_txt(top_dir):
                     else:
                         name = None
         if error:
-            logging.error('Error: %s\n%s (skipped)' % (error, path))
+            logging.error('Error: {0}\n{1} (skipped)'.format(error, path))
             return None
     return katochan if katochan['name'] else None
 
 def read_descript_txt(top_dir):
-    path = os.path.join(top_dir, 'descript.txt')
+    path = os.path.join(top_dir, b'descript.txt')
     if os.access(path, os.R_OK):
         return ninix.config.create_from_file(path)
     return None
 
 def read_install_txt(top_dir):
-    path = os.path.join(top_dir, 'install.txt')
+    path = os.path.join(top_dir, b'install.txt')
     if os.access(path, os.R_OK):
         return ninix.config.create_from_file(path)
     return None
 
 def read_alias_txt(top_dir):
-    path = os.path.join(top_dir, 'alias.txt')
+    path = os.path.join(top_dir, b'alias.txt')
     if os.access(path, os.R_OK):
         return ninix.alias.create_from_file(path)
     return None
 
 def find_makoto_dll(top_dir):
-    return 1 if os.access(os.path.join(top_dir, 'makoto.dll'), os.R_OK) else 0
+    return 1 if os.access(os.path.join(top_dir, b'makoto.dll'), os.R_OK) else 0
 
 def find_surface_set(top_dir):
-    desc = read_descript_txt(os.path.join(top_dir, 'ghost', 'master'))
+    desc = read_descript_txt(os.path.join(top_dir, b'ghost', b'master'))
+    default_sakura = desc.get('sakura.seriko.defaultsurface', '0')
+    default_kero = desc.get('kero.seriko.defaultsurface', '10')
     shell_name = desc.get('name') if desc else None
     if not shell_name:
         inst = read_install_txt(top_dir)
         if inst:
             shell_name = inst.get('name')
-    surface_set = []
-    shell_dir = os.path.join(top_dir, 'shell')
+    surface_set = OrderedDict()
+    shell_dir = os.path.join(top_dir, b'shell')
     for name, desc, subdir in find_surface_dir(shell_dir):
         surface_dir = os.path.join(shell_dir, subdir)
         surface_info, alias, tooltips = read_surface_info(surface_dir)
         if surface_info and \
-           'surface0' in surface_info and 'surface10' in surface_info:
+           'surface{}'.format(default_sakura) in surface_info and \
+           'surface{}'.format(default_kero) in surface_info:
             if alias is None:
                 alias = read_alias_txt(surface_dir)
-            surface_set.append((name, surface_dir, desc, alias, surface_info,
-                                tooltips))
+            surface_set[subdir] = (name, surface_dir, desc, alias,
+                                   surface_info, tooltips)
     return shell_name, surface_set
 
 def find_surface_dir(top_dir):
     buf = []
-    path = os.path.join(top_dir, 'surface.txt')
+    path = os.path.join(top_dir, b'surface.txt')
     if os.path.exists(path):
         config = ninix.config.create_from_file(path)
         for name, subdir in config.items():
@@ -360,7 +385,7 @@ def find_surface_dir(top_dir):
             buf.append((name, desc, subdir))
     return buf
 
-re_surface = re.compile('surface([0-9]+)\.(png|dgp|ddp)')
+re_surface = re.compile(b'surface([0-9]+)\.(png|dgp|ddp)')
 
 def read_surface_info(surface_dir):
     surface = {}
@@ -369,10 +394,10 @@ def read_surface_info(surface_dir):
     except OSError:
         filelist = []
     filename_alias = {}
-    path = os.path.join(surface_dir, 'alias.txt')
+    path = os.path.join(surface_dir, b'alias.txt')
     if os.path.exists(path):
         dic = ninix.alias.create_from_file(path)
-        for basename, alias in dic.iteritems():
+        for basename, alias in dic.items():
             if basename.startswith('surface'):
                 filename_alias[alias] = basename
     # find png image and associated configuration file
@@ -380,7 +405,7 @@ def read_surface_info(surface_dir):
         basename, ext = os.path.splitext(filename)
         if basename in filename_alias:
             match = re_surface.match(
-                ''.join((filename_alias[basename], ext)))
+                b''.join((os.fsencode(filename_alias[basename]), ext)))
         else:
             match = re_surface.match(filename)
         if not match:
@@ -389,12 +414,12 @@ def read_surface_info(surface_dir):
         if not os.access(img, os.R_OK):
             continue
         key = ''.join(('surface', str(int(match.group(1)))))
-        txt = os.path.join(surface_dir, ''.join((basename, 's.txt')))
+        txt = os.path.join(surface_dir, b''.join((basename, b's.txt')))
         if os.access(txt, os.R_OK):
             config = ninix.config.create_from_file(txt)
         else:
             config = ninix.config.null_config()
-        txt = os.path.join(surface_dir, ''.join((basename, 'a.txt')))
+        txt = os.path.join(surface_dir, b''.join((basename, b'a.txt')))
         if os.access(txt, os.R_OK):
             config.update(ninix.config.create_from_file(txt))
         surface[key] = (img, config)
@@ -408,69 +433,73 @@ def read_surface_info(surface_dir):
             tooltips = config
         elif key.startswith('surface'):
             try:
-                img, null_config = surface[key]
+                img, prev_config = surface[key]
+                prev_config.update(config)
+                config = prev_config
             except KeyError:
                 img = None
             surface[key] = (img, config)
     # find surface elements
-    for key, (img, config) in surface.items():
+    for key, (img, config) in list(surface.items()):
         for key, method, filename, x, y in list_surface_elements(config):
             filename = filename.lower()
             basename, ext = os.path.splitext(filename)
             if basename not in surface:
-                surface[basename] = (os.path.join(surface_dir, filename),
+                surface[basename] = (os.path.join(surface_dir, os.fsencode(filename)),
                                      ninix.config.null_config())
     return surface, alias, tooltips
 
 def read_surfaces_txt(surface_dir):
     config_list = []
-    path = os.path.join(surface_dir, 'surfaces.txt')
+    path = os.path.join(surface_dir, b'surfaces.txt')
     try:
-        with open(path) as f:
+        with open(path, 'rb') as f:
             alias_buffer = []
             tooltips = {}
-            charset = 'Shift_JIS'
-            while 1:
-                line = f.readline()
-                if not line:
-                    break
-                if line.startswith('#') or line.startswith('//'):
+            charset = 'CP932'
+            buf = []
+            key = None
+            opened = False
+            if f.read(3) == codecs.BOM_UTF8:
+                charset = 'UTF-8'
+            else:
+                f.seek(0) # rewind
+            for line in f:
+                if line.startswith(b'#') or line.startswith(b'//'):
                     continue
-                key = line.strip()
-                if key.startswith('charset'):
+                if charset == 'CP932':
+                    # '\x81\x40': full-width space in CP932(Shift_JIS)
+                    temp = line.replace(b'\x81\x40', b'').strip()
+                else:
+                    temp = line.strip()
+                if not temp:
+                    continue
+                if temp.startswith(b'charset'):
                     try:
-                        charset = line.split(',', 1)[1].strip()
+                        charset = str(line.split(b',', 1)[1].strip(), 'ascii')
                     except:
                         pass
                     continue
-                if not key:
-                    continue
-                try:
-                    while 1:
-                        line = f.readline()
-                        if not line:
-                            raise ValueError, 'unexpected end of file'
-                        line = line.replace('\x81\x40', '').strip()
-                        if not line:
-                            continue
-                        elif line == '{':
-                            break
-                        key = line # ignore the preceding key
-                    buf = []
-                    while 1:
-                        line = f.readline()
-                        if not line:
-                            raise ValueError, 'unexpected end of file'
-                        line = line.replace('\x81\x40', '').strip()
-                        if not line:
-                            continue
-                        elif line == '}':
-                            break
-                        buf.append(line)
+                if key is None:
+                    if temp.endswith(b'{'):
+                        key = str(temp[:-1], charset)
+                        opened = True
+                    else:
+                        key = str(temp, charset)
+                elif temp == b'{':
+                    opened = True
+                elif temp.endswith(b'}'):
+                    if temp[:-1]:
+                        buf.append(temp[:-1])
+                    if not opened:
+                        logging.error(
+                            'syntax error: unbalnced "}" in surfaces.txt.')
                     if key in ['sakura.surface.alias', 'kero.surface.alias']:
                         alias_buffer.append(key)
                         alias_buffer.append('{')
-                        alias_buffer.extend(buf)
+#                        alias_buffer.extend(buf)
+                        for line in buf:
+                            alias_buffer.append(str(line, charset))
                         alias_buffer.append('}')
                     elif key.endswith('.tooltips'):
                         try:
@@ -479,20 +508,31 @@ def read_surfaces_txt(surface_dir):
                             pass
                         value = {}
                         for line in buf:
-                            line = line.split(',', 1)
-                            region, text = [unicode(s.strip(), charset, 'ignore') for s in line]
+                            line = line.split(b',', 1)
+                            region, text = [str(s.strip(), charset) for s in line]
                             value[region] = text
                             tooltips[key] = value
                     elif key.startswith('surface'):
-                        try:
-                            key = ''.join((key[:7], str(int(key[7:]))))
-                        except ValueError:
-                            pass
-                        config_list.append((key, ninix.config.create_from_buffer(buf)))
-                except ValueError, error:
-                    logging.error(
-                        '%s: %s (parsing not completed)' % (path, error))
-                    break
+                        keys = key.split(',')
+                        for key in keys:
+                            if not key:
+                                continue
+                            if key.startswith('surface'):
+                                try:
+                                    key = ''.join((key[:7], str(int(key[7:]))))
+                                except ValueError:
+                                    pass
+                            else:
+                                try:
+                                    key = ''.join(('surface', str(int(key))))
+                                except ValueError:
+                                    pass
+                            config_list.append((key, ninix.config.create_from_buffer(buf, charset)))
+                    buf = []
+                    key = None
+                    opened = False
+                else:
+                    buf.append(temp)
     except IOError:
         return config_list
     if alias_buffer:
@@ -513,13 +553,13 @@ def list_surface_elements(config):
             y = int(y)
         except ValueError:
             logging.error(
-                'invalid element spec for %s: %s' % (key, config[key]))
+                'invalid element spec for {0}: {1}'.format(key, config[key]))
             continue
         buf.append((key, method, filename, x, y))
     return buf
 
-re_balloon = re.compile('balloon([skc][0-9]+)\.(png)')
-re_annex   = re.compile('(arrow[01]|sstp)\.(png)')
+re_balloon = re.compile(b'balloon([skc][0-9]+)\.(png)')
+re_annex   = re.compile(b'(arrow[01]|sstp)\.(png)')
 
 def read_balloon_info(balloon_dir):
     balloon = {}
@@ -532,13 +572,13 @@ def read_balloon_info(balloon_dir):
         if not match:
             continue
         img = os.path.join(balloon_dir, filename)
-        if match.group(2) != 'png' and \
-           os.access(''.join((img[-3:], 'png')), os.R_OK):
+        if match.group(2) != b'png' and \
+           os.access(b''.join((img[-3:], b'png')), os.R_OK):
                 continue
         if not os.access(img, os.R_OK):
             continue
-        key = match.group(1)
-        txt = os.path.join(balloon_dir, 'balloon%ss.txt' % key)
+        key = os.fsdecode(match.group(1))
+        txt = os.path.join(balloon_dir, os.fsencode('balloon{0}s.txt'.format(key)))
         if os.access(txt, os.R_OK):
             config = ninix.config.create_from_file(txt)
         else:
@@ -551,61 +591,139 @@ def read_balloon_info(balloon_dir):
         img = os.path.join(balloon_dir, filename)
         if not os.access(img, os.R_OK):
             continue
-        key = match.group(1)
+        key = os.fsdecode(match.group(1))
         config = ninix.config.null_config()
         balloon[key] = (img, config)
     return balloon
 
+def read_plugin_txt(src_dir):
+    path = os.path.join(src_dir, b'plugin.txt')
+    try:
+        with open(path, 'rb') as f:
+            charset = 'UTF-8' # default
+            standard = 0.0
+            plugin_name = startup = None
+            menu_items = []
+            error = None
+            lineno = 0
+            if f.read(3) == codecs.BOM_UTF8:
+                charset = 'UTF-8' # XXX
+            else:
+                f.seek(0) # rewind
+            for line in f:
+                lineno += 1
+                if not line.strip() or line.startswith(b'#'):
+                    continue
+                if b':' not in line:
+                    error = 'line {0:d}: syntax error'.format(lineno)
+                    break
+                name, value = [x.strip() for x in line.split(b':', 1)]
+                if name == b'charset':
+                    charset = str(value, 'ascii')
+                elif name == b'standard':
+                    standard = float(value)
+                elif name == b'name':
+                    plugin_name = str(value, charset, 'ignore')
+                elif name == b'startup':
+                    startup_list = str(value, charset).split(',')
+#                    startup_list[0] = os.path.join(plugin_dir, startup_list[0])
+                    if not os.path.exists(os.path.join(src_dir, startup_list[0])):
+                        error = 'line {0:d}: invalid program name'.format(lineno)
+                        break
+                    startup = startup_list
+                elif name == b'menuitem':
+                    menuitem_list = str(value, charset, 'ignore').split(',')
+                    if len(menuitem_list) < 2:
+                        error = 'line {0:d}: syntax error'.format(lineno)
+                        break
+#                    menuitem_list[1] = os.path.join(plugin_dir, menuitem_list[1])
+                    if not os.path.exists(os.path.join(src_dir, os.fsencode(menuitem_list[1]))):
+                        error = 'line {0:d}: invalid program name'.format(lineno)
+                        break
+                    menu_items.append((menuitem_list[0], menuitem_list[1:]))
+                elif name == b'directory':
+                    plugin_dir = os.fsencode(str(value, charset, 'ignore'))
+                else:
+                    error = 'line {0:d}: syntax error'.format(lineno)
+                    break
+            else:
+                if plugin_name is None:
+                    error = "the 'name' header field is required"
+                elif not startup and not menu_items:
+                    error = "either 'startup' or 'menuitem' header field is required"
+                elif standard < PLUGIN_STANDARD[0] or \
+                        standard > PLUGIN_STANDARD[1]:
+                    error = "standard version mismatch"
+    except IOError:
+        return None
+    if error:
+        sys.stderr.write('Error: {0}\n{1} (skipped)\n'.format(error, path))
+        return None
+    return plugin_name, plugin_dir, startup, menu_items
+
 
 ###   TEST   ###
 
 def test():
+    head, tail = os.path.split(sys.path[0])
+    sys.path[0] = head # XXX
     import locale
     locale.setlocale(locale.LC_ALL, '')
     config = load_config()
     if config is None:
-        raise SystemExit, 'Home directory not found.\n'
-    ghosts, balloons, nekoninni, katochan, kinoko = config
+        raise SystemExit('Home directory not found.\n')
+    ghosts, balloons, plugins, nekoninni, katochan, kinoko = config
     # ghosts
-    for desc, shiori_dir, use_makoto, surface_set, prefix, shiori_dll, shiori_name in ghosts:
-        print 'GHOST', '=' * 50
-        print prefix
-        print str(desc).encode('utf-8', 'ignore')
-        print shiori_dir
-        print shiori_dll
-        print shiori_name
-        print 'use_makoto =', use_makoto
+    for key in ghosts:
+        desc, shiori_dir, use_makoto, surface_set, prefix, shiori_dll, shiori_name = ghosts[key]
+        print('GHOST', '=' * 50)
+        print(prefix)
+        print(str(desc))
+        print(shiori_dir)
+        print(shiori_dll)
+        print(shiori_name)
+        print('use_makoto =', use_makoto)
         if surface_set:
-            for name, surface_dir, desc, alias, surface, tooltips in surface_set:
-                print '-' * 50
-                print 'surface:', name.encode('utf-8', 'ignore')
-                print str(desc).encode('utf-8', 'ignore')
-                for k, v in surface.iteritems():
-                    print k, '=', v[0]
-                    print str(v[1]).encode('utf-8', 'ignore')
+            for name, surface_dir, desc, alias, surface, tooltips in surface_set.values():
+                print('-' * 50)
+                print('surface:', name)
+                print(str(desc))
+                for k, v in surface.items():
+                    print(k, '=', v[0])
+                    print(str(v[1]))
                 if alias:
                     buf = []
-                    for k, v in alias.iteritems():
+                    for k, v in alias.items():
                         if k in ['sakura.surface.alias', 'kero.surface.alias']:
-                            print ''.join((k, ':'))
-                            for alias_id, alias_list in v.iteritems():
-                                print alias_id, \
-                                      ''.join(('= [', ', '.join(alias_list), ']'))
-                            print
+                            print(''.join((k, ':')))
+                            for alias_id, alias_list in v.items():
+                                print(alias_id, \
+                                      ''.join(('= [', ', '.join(alias_list), ']')))
+                            print()
                         else:
                             buf.append((k, v))
                     if buf:
-                        print 'filename alias:'
+                        print('filename alias:')
                         for k, v in buf:
-                            print k, '=', v
-                        print
+                            print(k, '=', v)
+                        print()
     # balloons
-    for desc, balloon in balloons:
-        print 'BALLOON', '=' * 50
-        print str(desc).encode('utf-8', 'ignore')
-        for k, v in balloon.iteritems():
-            print k, '=', v[0]
-            print str(v[1]).encode('utf-8', 'ignore')
+    for key in balloons:
+        desc, balloon = balloons[key]
+        print('BALLOON', '=' * 50)
+        print(str(desc))
+        for k, v in balloon.items():
+            print(k, '=', v[0])
+            print(str(v[1]))
+    # plugins
+    for plugin_name, plugin_dir, startup, menu_items in plugins:
+        print('PLUGIN', '=' * 50)
+        print('name =', plugin_name)
+        if startup:
+            print('startup =', ''.join(('["', '", "'.join(startup), '"]')))
+        for label, argv in menu_items:
+            print("menuitem '{0}' =".format(label),)
+            print(''.join(('["', '", "'.join(argv), '"]')))
     ## FIXME
     # kinoko
     # nekoninni
