@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #  Copyright (C) 2001-2004 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2004-2011 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2004-2013 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -15,17 +15,22 @@ import codecs
 import os
 import re
 import time
-import urllib
+import urllib.request
+import urllib.parse
 import webbrowser
+import logging
 
 from xml.dom import pulldom
 
-import gtk
-import gobject
+from gi.repository import Gtk
+from gi.repository import Gdk
+from gi.repository import Pango
+from gi.repository import GObject
 import cairo
 
 import ninix.home
 import ninix.pix
+import ninix.install
 
 # 注意:
 # - このURLを本ゴーストマネージャクローン以外の用途で使う場合は,
@@ -72,14 +77,13 @@ class Catalog_xml:
         last_update = self.last_update
         self.last_update = time.strftime('%Y-%m-%d %H:%M:%S')
         if self.cgi:
-            priority = self.cgi.keys()
-            priority.sort()
+            priority = sorted(self.cgi.keys())
             url = self.cgi[priority[-1]][-1]
         else:
             url = MASTERLIST_URL
         try:
-            f = urllib.urlopen(url, urllib.urlencode(
-                {'time': '"%s"' % last_update, 'charset': 'UTF-8'}))
+            f = urllib.request.urlopen(url, urllib.parse.urlencode(
+                {'time': '"{0}"'.format(last_update), 'charset': 'UTF-8'}))
         except:
             return ## FIXME
         for _ in self.import_from_fileobj(f):
@@ -90,14 +94,14 @@ class Catalog_xml:
     # private methods
     def load_MasterList(self):
         try:
-            with open(os.path.join(self.datadir, 'MasterList.xml'), 'r') as f:
+            with open(os.path.join(self.datadir, b'MasterList.xml'), 'r') as f:
                 for _ in self.import_from_fileobj(f):
                     pass
         except IOError:
             return
 
     def save_MasterList(self):
-        with open(os.path.join(self.datadir, 'MasterList.xml'), 'w') as f:
+        with open(os.path.join(self.datadir, b'MasterList.xml'), 'w') as f:
             self.export_to_fileobj(f)
 
     def get_encoding(self, line):
@@ -117,7 +121,7 @@ class Catalog_xml:
         try:
             codecs.lookup(encoding)
         except:
-            raise SystemExit, 'Unsupported encoding %s' % repr(encoding)
+            raise SystemExit('Unsupported encoding {0}'.format(repr(encoding)))
         nest = 0
         new_entry = {}
         set_id = None
@@ -130,7 +134,6 @@ class Catalog_xml:
         for line in fileobj:
             yield
             assert nest >= 0
-            line = unicode(line, encoding, 'ignore').encode('utf-8')
             if not line:
                 continue
             m = re_list.search(line)
@@ -199,34 +202,30 @@ class Catalog_xml:
                 text = text.replace('>', '&gt;')
                 text = text.replace('"', '&quot;')
                 text = text.replace('\'', '&apos;')
-                text = text.encode('UTF-8')
             else:
-                text = '' # fileobj.write('  <%s/>\n' % key)
-            fileobj.write('  <%s>%s</%s>\n' % (key, text, key))
+                text = '' # fileobj.write('  <{0}/>\n'.format(key))
+            fileobj.write('  <{0}>{1}</{0}>\n'.format(key, text))
 
     def export_to_fileobj(self, fileobj):
         fileobj.write('<?xml version="1.0" encoding="UTF-8" ?>\n')
         fileobj.write('<GhostList>\n')
-        fileobj.write('<LastUpdate>%s</LastUpdate>\n' % self.last_update)
+        fileobj.write('<LastUpdate>{0}</LastUpdate>\n'.format(self.last_update))
         for key in ['SakuraPreviewBaseUrl', 'KeroPreviewBaseUrl',
                     'ArcMD5BaseUrl', 'NGMUpdateBaseUrl']:
             if key in self.url:
-                fileobj.write('<%s>%s</%s>\n' % (key, self.url[key], key))
+                fileobj.write('<{0}>{1}</{0}>\n'.format(key, self.url[key]))
             else:
-                fileobj.write('<%s></%s>\n' % (key, key)) # fileobj.write('<%s/>\n' % key)
-        fileobj.write('<NGMVersion>%s</NGMVersion>\n' % self.version)
-        key_list = self.cgi.keys()
-        key_list.sort()
+                fileobj.write('<{0}></{0}>\n'.format(key)) # fileobj.write('<{0}/>\n'.format(key))
+        fileobj.write('<NGMVersion>{0}</NGMVersion>\n'.format(self.version))
+        key_list = sorted(self.cgi.keys())
         key_list.reverse()
         for priority in key_list:
             for url in self.cgi[priority]:
                 fileobj.write(
-                    '<RequestCgiUrl Priority="%d">%s</RequestCgiUrl>\n' % \
-                    (priority, url))
-        ids = self.data.keys()
-        ids.sort()
+                    '<RequestCgiUrl Priority="{0:d}">{1}</RequestCgiUrl>\n'.format(priority, url))
+        ids = sorted(self.data.keys())
         for set_id in ids:
-            fileobj.write('<FileSet ID="%s">\n' % set_id)
+            fileobj.write('<FileSet ID="{0}">\n'.format(set_id))
             entry = self.data[set_id]
             self.dump_entry(entry, fileobj)
             fileobj.write('</FileSet>\n')
@@ -238,56 +237,43 @@ class Catalog(Catalog_xml):
     TYPE = ['Sakura', 'Kero']
 
     def image_filename(self, set_id, side):
-        p = '%s_%d.png' % (self.TYPE[side], set_id)
-        d = os.path.join(self.datadir, p)
+        p = '{0}_{1:d}.png'.format(self.TYPE[side], set_id)
+        d = os.path.join(self.datadir, os.fsencode(p))
         return d if os.path.exists(d) else None
 
     def retrieve_image(self, set_id, side, updatehook):
-        p = '%s_%d.png' % (self.TYPE[side], set_id)
-        d = os.path.join(self.datadir, p)
+        p = '{0}_{1:d}.png'.format(self.TYPE[side], set_id)
+        d = os.path.join(self.datadir, os.fsencode(p))
         if not os.path.exists(d):
             url = self.url['SakuraPreviewBaseUrl'] if side == 0 else \
                 self.url['KeroPreviewBaseUrl']
             try:
-                urllib.urlretrieve(''.join((url, p)), d, updatehook)
+                urllib.request.urlretrieve(''.join((url, p)), d, updatehook)
             except:
                 return ## FIXME
 
 
-#class InstallationManager(list):
-#
-#    def __init__(self, ngm):
-#        self.master = ngm
-#
-#    def set_current_ghost(self, set_id):
-#        self.current_ghost = set_id
-#
-#    def install_ghost(self, dialog):
-#        a = self.master.ui.catalog.get(self.current_ghost, 'ArchiveUrl')
-#        os.system("%s '%s'" % (self.master.ninix_install_cmd, a))
-
-
 class SearchDialog:
 
-    def __init__(self, callback):
-        self.callback = callback
-        self.dialog = gtk.Dialog()
-        self.dialog.connect('delete_event', self.cancel)
+    def __init__(self):
+        self.request_parent = lambda *a: None # dummy
+        self.dialog = Gtk.Dialog()
+        self.dialog.connect('delete_event', lambda *a: True) # XXX
         self.dialog.set_modal(True)
-        self.dialog.set_position(gtk.WIN_POS_CENTER)
-        label = gtk.Label(_('Search for'))
-        self.dialog.vbox.pack_start(label)
-        self.pattern_entry = gtk.Entry()
+        self.dialog.set_position(Gtk.WindowPosition.CENTER)
+        label = Gtk.Label(label=_('Search for'))
+        content_area = self.dialog.get_content_area()
+        content_area.add(label)
+        self.pattern_entry = Gtk.Entry()
         self.pattern_entry.set_size_request(300, -1)
-        self.dialog.vbox.pack_start(self.pattern_entry)
-        self.dialog.vbox.show_all()
-        self.ok_button = gtk.Button(_('OK'))
-        self.ok_button.connect('clicked', self.ok)
-        self.dialog.action_area.pack_start(self.ok_button)
-        button = gtk.Button(_('Cancel'))
-        button.connect('clicked', self.cancel)
-        self.dialog.action_area.pack_start(button)
-        self.dialog.action_area.show_all()
+        content_area.add(self.pattern_entry)
+        content_area.show_all()
+        self.dialog.add_button(Gtk.STOCK_OK, Gtk.ResponseType.OK)
+        self.dialog.add_button(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL)
+        self.dialog.connect('response', self.response)
+
+    def set_responsible(self, request_method):
+        self.request_parent = request_method
 
     def set_pattern(self, text):
         self.pattern_entry.set_text(text)
@@ -303,20 +289,27 @@ class SearchDialog:
             self.set_pattern(default)
         self.dialog.show()
 
-    def ok(self, widget, event=None):
-        word = unicode(self.get_pattern(), 'utf-8')
-        self.callback['search'](word)
+    def ok(self):
+        word = self.get_pattern()
+        self.request_parent('NOTIFY', 'search', word)
         self.hide()
-        return True
 
-    def cancel(self, widget, event=None):
+    def cancel(self):
         self.hide()
+
+    def response(self, widget, response):
+        func = {Gtk.ResponseType.OK: self.ok,
+                Gtk.ResponseType.CANCEL: self.cancel,
+                Gtk.ResponseType.DELETE_EVENT: self.cancel,
+                }
+        func[response]()
         return True
 
 
 class UI:
-    def __init__(self, callback):
-        self.callback = callback
+
+    def __init__(self):
+        self.request_parent = lambda *a: None # dummy
         self.ui_info = '''
         <ui>
           <menubar name='MenuBar'>
@@ -357,7 +350,8 @@ class UI:
             ( 'Settings', None,
               _('Settings(_O)'), None,
               None,
-              lambda *a: self.callback['open_preference_dialog']() ),
+              lambda *a: self.request_parent(
+                    'NOTIFY', 'open_preference_dialog') ),
             ( 'DB Network Update', None,
               _('DB Network Update(_N)'), None,
               None,
@@ -369,54 +363,71 @@ class UI:
             ( 'Mask', None,
               _('Mask(_M)'), None,
               None,
-              lambda *a: self.callback['open_mask_dialog']() ),
+              lambda *a: self.request_parent(
+                    'NOTIFY', 'open_mask_dialog') ),
             ( 'Reset to Default', None,
               _('Reset to Default(_Y)'), None,
               None,
-              lambda *a: self.callback['reset_to_default']() ),
+              lambda *a: self.request_parent(
+                    'NOTIFY', 'reset_to_default') ),
             ( 'Show All', None,
               _('Show All(_Z)'), None,
               None,
-              lambda *a: self.callback['show_all']() ),
+              lambda *a: self.request_parent(
+                    'NOTIFY', 'show_all') ),
             )
         self.opened = 0
         self.textview = [None, None]
         self.darea = [None, None]
-        self.pixbuf = [None, None]
+        self.surface = [None, None]
         self.info = None
         self.button = {}
         self.url = {}
         self.search_word = ''
-        search_dialog_callback = {
-            'search': self.search,
-            }
-        self.search_dialog = SearchDialog(search_dialog_callback)
+        self.search_dialog = SearchDialog()
+        self.search_dialog.set_responsible(self.handle_request)
         self.create_dialog()
 
+    def set_responsible(self, request_method):
+        self.request_parent = request_method
+
+    def handle_request(self, event_type, event, *arglist, **argdict):
+        assert event_type in ['GET', 'NOTIFY']
+        handlers = {
+            }
+        handler = handlers.get(event, getattr(self, event, None))
+        if handler is None:
+            result = self.request_parent(
+                event_type, event, *arglist, **argdict)
+        else:
+            result = handler(*arglist, **argdict)
+        if event_type == 'GET':
+            return result
+
     def create_dialog(self):
-        self.window = gtk.Window()
+        self.window = Gtk.Window()
         self.window.set_title(_('Ghost Manager'))
         self.window.set_resizable(False)
         self.window.connect('delete_event', lambda *a: self.close())
-        self.window.set_position(gtk.WIN_POS_CENTER)
-        self.window.set_gravity(gtk.gdk.GRAVITY_CENTER)
-        actions = gtk.ActionGroup('Actions')
+        self.window.set_position(Gtk.WindowPosition.CENTER)
+        self.window.set_gravity(Gdk.Gravity.CENTER)
+        actions = Gtk.ActionGroup('Actions')
         actions.add_actions(self.entries)
-        ui = gtk.UIManager()
+        ui = Gtk.UIManager()
         ui.insert_action_group(actions, 0)
         self.window.add_accel_group(ui.get_accel_group())
         try:
             mergeid = ui.add_ui_from_string(self.ui_info)
-        except gobject.GError, msg:
-            print 'building menus failed: %s' % msg
-        vbox = gtk.VBox(False, 0)
+        except GObject.GError as msg:
+            logging.error('building menus failed: {0}'.format(msg))
+        vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
         self.window.add(vbox)
         vbox.show()
         vbox.pack_start(ui.get_widget('/MenuBar'), False, False, 0)
-        separator = gtk.HSeparator()
+        separator = Gtk.HSeparator()
         vbox.pack_start(separator, False, True, 0)
         separator.show()
-        hbox = gtk.HBox(False, 0)
+        hbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
         vbox.pack_start(hbox, False, True, 10)
         hbox.show()
         self.surface_area_sakura = self.create_surface_area(0)
@@ -425,30 +436,30 @@ class UI:
         hbox.pack_start(self.surface_area_kero, False, True, 10)
         self.info_area = self.create_info_area()
         hbox.pack_start(self.info_area, False, True, 10)
-        box = gtk.HButtonBox()
-        box.set_layout(gtk.BUTTONBOX_SPREAD)
+        box = Gtk.ButtonBox(orientation=Gtk.Orientation.HORIZONTAL)
+        box.set_layout(Gtk.ButtonBoxStyle.SPREAD)
         vbox.pack_start(box, False, True, 4)
         box.show()
-        button = gtk.Button(_('Previous'))
+        button = Gtk.Button(_('Previous'))
         button.connect('clicked', lambda b, w=self: w.show_previous())
         box.add(button)
         button.show()
         self.button['previous'] = button
-        button = gtk.Button(_('Next'))
+        button = Gtk.Button(_('Next'))
         button.connect('clicked', lambda b, w=self: w.show_next())
         box.add(button)
         button.show()
         self.button['next'] = button
-        self.statusbar = gtk.Statusbar()
+        self.statusbar = Gtk.Statusbar()
         vbox.pack_start(self.statusbar, False, True, 0)
         self.statusbar.show()
 
     def network_update(self):
         self.window.set_sensitive(False)
         def updatehook(*args):
-            while gtk.events_pending():
-                gtk.main_iteration()
-        self.callback['network_update'](updatehook)
+            while Gtk.events_pending():
+                Gtk.main_iteration()
+        self.request_parent('NOTIFY', 'network_update', updatehook)
         self.update()
         self.window.set_sensitive(True)
 
@@ -458,112 +469,112 @@ class UI:
     def search(self, word):
         if word:
             self.search_word = word
-            if self.callback['search'](word):
+            if self.request_parent('GET', 'search', word):
                 self.update()
             else:
                 pass ## FIXME
 
     def search_forward(self):
         if self.search_word:
-            if self.callback['search_forward'](self.search_word):
+            if self.request_parent('GET', 'search_forward', self.search_word):
                 self.update()
             else:
                 pass ## FIXME
 
     def show_next(self):
-        self.callback['next']()
+        self.request_parent('NOTIFY', 'go_next')
         self.update()
 
     def show_previous(self):
-        self.callback['previous']()
+        self.request_parent('NOTIFY', 'previous')
         self.update()
 
     def create_surface_area(self, side):
         assert side in [0, 1]
-        vbox = gtk.VBox(False, 0)
+        vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
         vbox.show()
-        textview = gtk.TextView()
+        textview = Gtk.TextView()
         textview.set_editable(False)
         textview.set_size_request(128, 16)
         vbox.pack_start(textview, False, True, 0)
         textview.show()
         self.textview[side] = textview
-        darea = gtk.DrawingArea()
+        darea = Gtk.DrawingArea()
         vbox.pack_start(darea, False, True, 0)
+        darea.set_events(Gdk.EventMask.EXPOSURE_MASK)
+        darea.connect('draw', self.redraw, side)
         darea.show()
-        darea.set_events(gtk.gdk.EXPOSURE_MASK)
-        darea.connect('expose_event', self.redraw, side)
         self.darea[side] = darea
         return vbox
 
-    def redraw(self, darea, event, side):
-        cr = darea.window.cairo_create()
-        cr.save()
-        cr.set_operator(cairo.OPERATOR_CLEAR)
-        cr.paint()
-        cr.restore()
-        if self.pixbuf[side] is not None:
-            cr.set_source_pixbuf(self.pixbuf[side], 0, 0)
+    def redraw(self, widget, cr, side):
+        if self.surface[side] is not None:
+            cr.set_source_surface(self.surface[side], 0, 0)
+            cr.set_operator(cairo.OPERATOR_SOURCE)
+            cr.paint()
+        else:
+            cr.set_operator(cairo.OPERATOR_CLEAR)
             cr.paint()
-        del cr
 
     def update_surface_area(self):
         for side in [0, 1]:
-            name = self.callback['get']('SakuraName') if side == 0 else \
-                self.callback['get']('KeroName')
+            target = 'SakuraName' if side == 0 else 'KeroName'
+            name = self.request_parent('GET', 'get', target)
             textbuffer = self.textview[side].get_buffer()
             textbuffer.set_text(name)
-            filename = self.callback['get_image_filename'](side)
+            filename = self.request_parent('GET', 'get_image_filename', side)
             darea = self.darea[side]
             darea.realize()
             if filename is not None:
                 try:
-                    pixbuf = ninix.pix.create_pixbuf_from_file(filename)
+                    surface = ninix.pix.create_surface_from_file(filename)
                 except:
-                    pixbuf = None
+                    surface = None
                 else:
-                    w = pixbuf.get_width()
-                    h = pixbuf.get_height()
+                    w = surface.get_width()
+                    h = surface.get_height()
                     darea.set_size_request(w, h)
-                    mask = gtk.gdk.Pixmap(None, w, h, 1)
-                    pixbuf.render_threshold_alpha(
-                        mask, 0, 0, 0, 0, w, h, 1)
-                    darea.window.shape_combine_mask(mask, 0, 0)
+                    region = Gdk.cairo_region_create_from_surface(surface)
+                    darea.get_window().shape_combine_region(region, 0, 0)
             else:
-                pixbuf = None
-            self.pixbuf[side] = pixbuf
+                surface = None
+            self.surface[side] = surface
             darea.queue_draw()
 
     def create_info_area(self):
-        vbox = gtk.VBox(False, 0)
+        vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
         vbox.show()
-        hbox = gtk.HBox(False, 0)
-        box = gtk.HButtonBox()
-        box.set_layout(gtk.BUTTONBOX_SPREAD)
+        hbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
+        box = Gtk.ButtonBox(orientation=Gtk.Orientation.HORIZONTAL)
+        box.set_layout(Gtk.ButtonBoxStyle.SPREAD)
         box.show()
-        button = gtk.Button(_('Install'))
-        button.connect('clicked', lambda b, w=self: w.callback['install_current']())
+        button = Gtk.Button(_('Install'))
+        button.connect(
+            'clicked',
+            lambda b, w=self: w.request_parent('NOTIFY', 'install_current'))
         box.add(button)
         button.show()
         self.button['install'] = button
-        button = gtk.Button(_('Update'))
-        button.connect('clicked', lambda b, w=self: w.callback['update_current']())
+        button = Gtk.Button(_('Update'))
+        button.connect(
+            'clicked',
+            lambda b, w=self: w.request_parent('NOTIFY', 'update_current'))
         box.add(button)
         button.show()
         self.button['update'] = button
         hbox.pack_start(box, True, True, 10)
-        vbox2 = gtk.VBox(False, 0)
+        vbox2 = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
         hbox.pack_start(vbox2, False, True, 0)
         vbox2.show()
-        button = gtk.Button('') # with GtkLabel
-        button.set_relief(gtk.RELIEF_NONE)
+        button = Gtk.Button('') # with GtkLabel
+        button.set_relief(Gtk.ReliefStyle.NONE)
         self.url['HP'] = [None, button.get_child()]
         vbox2.pack_start(button, False, True, 0)
         button.connect(
             'clicked', lambda b: webbrowser.open(self.url['HP'][0]))
         button.show()
-        button = gtk.Button('')
-        button.set_relief(gtk.RELIEF_NONE)
+        button = Gtk.Button('')
+        button.set_relief(Gtk.ReliefStyle.NONE)
         button.set_use_underline(True)
         self.url['Public'] = [None, button.get_child()]
         vbox2.pack_start(button, False, True, 0)
@@ -572,7 +583,7 @@ class UI:
         button.show()
         vbox.pack_start(hbox, False, True, 0)
         hbox.show()
-        textview = gtk.TextView()
+        textview = Gtk.TextView()
         textview.set_editable(False)
         textview.set_size_request(256, 144)
         vbox.pack_start(textview, False, True, 0)
@@ -588,44 +599,52 @@ class UI:
                      (_('Version:'), 'Version'),
                      (_('AIName:'), 'AIName')]
         text = ''
-        text = ''.join((text, self.callback['get']('Name'), '\n'))
+        text = ''.join((text, self.request_parent('GET', 'get', 'Name'), '\n'))
         for item in info_list:
-            text = ''.join((text, item[0], self.callback['get'](item[1]), '\n'))
+            text = ''.join((text, item[0],
+                            self.request_parent('GET', 'get', item[1]), '\n'))
         text = ''.join((text,
-                        self.callback['get']('SakuraName'),
-                        _('SurfaceList:'), self.callback['get']('SakuraSurfaceList'),
+                        self.request_parent('GET', 'get', 'SakuraName'),
+                        _('SurfaceList:'),
+                        self.request_parent('GET', 'get', 'SakuraSurfaceList'),
                         '\n'))
         text = ''.join((text,
-                        self.callback['get']('KeroName'),
-                        _('SurfaceList:'), self.callback['get']('KeroSurfaceList'),
+                        self.request_parent('GET', 'get', 'KeroName'),
+                        _('SurfaceList:'),
+                        self.request_parent('GET', 'get', 'KeroSurfaceList'),
                         '\n'))
         textbuffer = self.info.get_buffer()
         textbuffer.set_text(text)
-        url = self.callback['get']('HPUrl')
-        text = self.callback['get']('HPTitle')
+        url = self.request_parent('GET', 'get', 'HPUrl')
+        text = self.request_parent('GET', 'get', 'HPTitle')
         self.url['HP'][0] = url
         label = self.url['HP'][1]
-        label.set_markup('<span foreground="blue">%s</span>' % text)
-        url = self.callback['get']('PublicUrl')
-        text = ''.join((self.callback['get']('Name'), _(' Web Page')))
+        label.set_markup('<span foreground="blue">{0}</span>'.format(text))
+        url = self.request_parent('GET', 'get', 'PublicUrl')
+        text = ''.join((self.request_parent('GET', 'get', 'Name'),
+                        _(' Web Page')))
         self.url['Public'][0] = url
         label = self.url['Public'][1]
-        label.set_markup('<span foreground="blue">%s</span>' % text)        
-        target_dir = os.path.join(self.callback['get_home_dir'](), 'ghost',
-                                  self.callback['get']('InstallDir'))
+        label.set_markup('<span foreground="blue">{0}</span>'.format(text))
+        target_dir = os.path.join(
+            os.fsencode(self.request_parent('GET', 'get_home_dir')),
+            b'ghost',
+            os.fsencode(self.request_parent('GET', 'get', 'InstallDir')))
         self.button['install'].set_sensitive(
             bool(not os.path.isdir(target_dir) and
-                 self.callback['get']('ArchiveUrl') != 'No data'))
+                 self.request_parent('GET', 'get', 'ArchiveUrl') != 'No data'))
         self.button['update'].set_sensitive(
             bool(os.path.isdir(target_dir) and
-                 self.callback['get']('GhostType') == 'ゴースト' and
-                 self.callback['get']('UpdateUrl') != 'No data'))
+                 self.request_parent('GET', 'get', 'GhostType') == 'ゴースト' and
+                 self.request_parent('GET', 'get', 'UpdateUrl') != 'No data'))
 
     def update(self):
         self.update_surface_area()
         self.update_info_area()
-        self.button['next'].set_sensitive(bool(self.callback['exist_next']()))
-        self.button['previous'].set_sensitive(bool(self.callback['exist_previous']()))
+        self.button['next'].set_sensitive(
+            bool(self.request_parent('GET', 'exist_next')))
+        self.button['previous'].set_sensitive(
+            bool(self.request_parent('GET', 'exist_previous')))
 
     def show(self):
         if self.opened:
@@ -642,31 +661,32 @@ class UI:
 
 class NGM:
 
-    def __init__(self, callback):
-        self.callback = callback
+    def __init__(self):
+        self.request_parent = lambda *a: None # dummy
         self.current = 0
         self.opened = 0
         self.home_dir = ninix.home.get_ninix_home()
-        self.catalog = Catalog(os.path.join(self.home_dir, 'ngm/data'))
-        ui_callback = {
-            'open_preference_dialog': self.open_preference_dialog,
-            'open_mask_dialog': self.open_mask_dialog,
-            'reset_to_default': self.reset_to_default,
-            'show_all': self.show_all,
-            'network_update': self.network_update,
-            'search': self.search,
-            'search_forward': self.search_forward,
-            'next': self.next,
-            'previous': self.previous,
-            'get': self.get,
-            'get_image_filename': self.get_image_filename,
+        self.catalog = Catalog(os.path.join(self.home_dir, b'ngm/data'))
+        self.installer = ninix.install.Installer()
+        self.ui = UI()
+        self.ui.set_responsible(self.handle_request)
+
+    def set_responsible(self, request_method):
+        self.request_parent = request_method
+
+    def handle_request(self, event_type, event, *arglist, **argdict):
+        assert event_type in ['GET', 'NOTIFY']
+        handlers = {
             'get_home_dir': lambda *a: self.home_dir,
-            'exist_next': self.exist_next,
-            'exist_previous': self.exist_previous,
-            'install_current': self.install_current,
-            'update_current': self.update_current,
             }
-        self.ui = UI(ui_callback)
+        handler = handlers.get(event, getattr(self, event, None))
+        if handler is None:
+            result = self.request_parent(
+                event_type, event, *arglist, **argdict)
+        else:
+            result = handler(*arglist, **argdict)
+        if event_type == 'GET':
+            return result
 
     def get(self, element):
         if self.current in self.catalog.data:
@@ -686,7 +706,9 @@ class NGM:
                 for element in ['Name', 'SakuraName', 'KeroName',
                                 'Author', 'HPTitle']:
                     text = self.catalog.get(entry, element)
-                    if text and word in text:
+                    if not text:
+                        continue
+                    if word in text:
                         self.current = set_id
                         return True
             set_id += 1
@@ -713,14 +735,14 @@ class NGM:
     def show_all(self):
         pass
 
-    def next(self):
-        next = self.current + 1
-        if next < ID_LIMIT and next in self.catalog.data:
-            self.current = next
+    def go_next(self):
+        next_index = self.current + 1
+        if next_index < ID_LIMIT and next_index in self.catalog.data:
+            self.current = next_index
 
     def exist_next(self):
-        next = self.current + 1
-        return bool(next < ID_LIMIT and next in self.catalog.data)
+        next_index = self.current + 1
+        return bool(next_index < ID_LIMIT and next_index in self.catalog.data)
 
     def previous(self):
         previous = self.current - 1
@@ -736,11 +758,18 @@ class NGM:
     def show_dialog(self):
         self.ui.show()
 
-    def install_current(self): ## FIXME
-        pass
+    def install_current(self):
+        try:
+            filetype, target_dir = self.installer.install(
+                self.get('ArchiveUrl'), ninix.home.get_ninix_home())
+        except:
+            target_dir = None
+        assert filetype == 'ghost'
+        if target_dir is not None:
+            self.request_parent('NOTIFY', 'add_sakura', target_dir)
 
     def update_current(self): ## FIXME
-        self.callback['update_sakura'](self.get('Name'), 'NGM')
+        self.request_parent('NOTIFY', 'update_sakura', self.get('Name'), 'NGM')
 
 
 if __name__ == '__main__':
