@@ -2,7 +2,7 @@
 #
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2012 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2013 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2003 by Shun-ichi TAHARA <jado@flowernet.gr.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -225,6 +225,9 @@ class Sakura:
         else:
             self.set_SSP_mode(0)
         self.last_script = None
+        self.status_icon = Gtk.StatusIcon()
+        self.status_icon.set_title(self.get_name(default=''))
+        self.status_icon.set_visible(False)
 
     def set_SSP_mode(self, flag): # XXX
         self.__sender = 'SSP' if flag else 'ninix-aya'
@@ -363,7 +366,10 @@ class Sakura:
             self.audio_loop = False
 
     def set_surface(self, desc, alias, surface, name, surface_dir, tooltips):
-        self.surface.new(desc, alias, surface, name, surface_dir, tooltips)
+        default_sakura = self.desc.get('sakura.seriko.defaultsurface', '0')
+        default_kero = self.desc.get('kero.seriko.defaultsurface', '10')
+        self.surface.new(desc, alias, surface, name, surface_dir, tooltips,
+                         default_sakura, default_kero)
         for side in range(2, self.char):
             default = self.desc.get('char{0:d}.seriko.defaultsurface'.format(side))
             self.surface.add_window(side, default)
@@ -660,7 +666,8 @@ class Sakura:
 
     ###   CALLBACK   ###
     def notify_start(self, init, vanished, ghost_changed,
-                     name, prev_name, prev_shell, path, last_script):
+                     name, prev_name, prev_shell, path, last_script,
+                     abend=None):
         if self.__temp_mode:
             default = None
         else:
@@ -670,9 +677,21 @@ class Sakura:
                 if not self.notify_event('OnFirstBoot', self.vanished_count,
                                          None, None, None, None, None, None,
                                          self.surface.name):
-                    self.notify_event('OnBoot', self.surface.name, default=default)
+                    if abend is not None:
+                        self.notify_event('OnBoot', self.surface.name,
+                                          None, None, None, None, None,
+                                          'halt', abend, default=default)
+                    else:
+                        self.notify_event('OnBoot', self.surface.name,
+                                          default=default)
             else:
-                self.notify_event('OnBoot', self.surface.name, default=default)
+                if abend is not None:
+                    self.notify_event('OnBoot', self.surface.name,
+                                      None, None, None, None, None,
+                                      'halt', abend, default=default)
+                else:
+                    self.notify_event('OnBoot', self.surface.name,
+                                      default=default)
             left, top, scrn_w, scrn_h = ninix.pix.get_workarea()
             self.notify_event('OnDisplayChange',
                               Gdk.Visual.get_best_depth(),
@@ -689,7 +708,12 @@ class Sakura:
                                    prev_name, None, None, None, None,
                                    pref_shell):
                 return
-            self.notify_event('OnBoot', self.surface.name, default=default)
+            if abend is not None:
+                self.notify_event('OnBoot', self.surface.name,
+                                  None, None, None, None, None, None,
+                                  'halt', abend, default=default)
+            else:
+                self.notify_event('OnBoot', self.surface.name, default=default)
         elif ghost_changed:
             if self.ghost_time == 0:
                 if self.notify_event('OnFirstBoot', self.vanished_count,
@@ -700,7 +724,12 @@ class Sakura:
                                    prev_name, None, None, None, None,
                                    prev_shell):
                 return
-            self.notify_event('OnBoot', self.surface.name, default=default)
+            if abend is not None:
+                self.notify_event('OnBoot', self.surface.name,
+                                  None, None, None, None, None,
+                                  'halt', abend, default=default)
+            else:
+                self.notify_event('OnBoot', self.surface.name, default=default)
         else:
             pass ## FIXME
 
@@ -853,8 +882,8 @@ class Sakura:
     reset_event = ['OnVanishSelecting', 'OnVanishCancel'] ## FIXME
 
     def notify_event(self, event, *arglist, **argdict):
-        #if self.time_critical_session:
-        #    return 0
+        if self.time_critical_session and event.startswith('OnMouse'):
+            return 0
         if event in self.reset_event:
             self.reset_script(1)
         for key in argdict:
@@ -1161,16 +1190,18 @@ class Sakura:
     def stand_by(self, reset_surface):
         self.balloon.hide_all()
         self.balloon.hide_sstp_message()
+        default_sakura = self.desc.get('sakura.seriko.defaultsurface', '0')
+        default_kero = self.desc.get('kero.seriko.defaultsurface', '10')
         if reset_surface:
             self.set_surface_default()
             self.balloon.set_balloon_default()
-        elif self.get_surface_id(0) != '0' or \
-             self.get_surface_id(1) != '10':
+        elif self.get_surface_id(0) != default_sakura or \
+             self.get_surface_id(1) != default_kero:
             self.__surface_life = random.randint(20, 30)
             ##logging.debug('surface_life = {0:d}'.format(self.__surface_life))
 
     def start(self, key, init, temp, vanished, ghost_changed,
-              prev_self_name, prev_name, prev_shell, last_script):
+              prev_self_name, prev_name, prev_shell, last_script, abend):
         if self.is_running():
             if temp:
                 self.enter_temp_mode()
@@ -1180,7 +1211,8 @@ class Sakura:
                     self.load_shiori()
                     self.notify_start(
                         init, vanished, ghost_changed,
-                        prev_self_name, prev_name, prev_shell, '', last_script)
+                        prev_self_name, prev_name, prev_shell,
+                        '', last_script, abend)
             return
         self.ghost_time = 0
         self.vanished_count = 0
@@ -1222,7 +1254,7 @@ class Sakura:
         self.start_time = time.time()
         self.notify_start(
             init, vanished, ghost_changed,
-            name, prev_name, prev_shell, surface_dir, last_script)
+            name, prev_name, prev_shell, surface_dir, last_script, abend)
         GLib.timeout_add(10, self.do_idle_tasks) # 10[ms]
 
     def restart(self):
@@ -1373,6 +1405,7 @@ class Sakura:
     def do_idle_tasks(self):
         if not self.__running:
             return False
+        self.request_parent('NOTIFY', 'update_working', self.get_name())
         if self.__temp_mode:
             self.process_script()
             if not self.busy() and \
@@ -1453,7 +1486,12 @@ class Sakura:
         self.quick_session = 0
         self.set_synchronized_session(reset=1)
         self.balloon.hide_all()
-        self.balloon.clear_text_all()
+        node = self.processed_script[0]
+        if node[0] == ninix.script.SCRIPT_TAG and node[1] == r'\C':
+            self.processed_script.pop(0)
+            self.script_position = node[-1]
+        else:
+            self.balloon.clear_text_all()
         self.balloon.set_balloon_default()
         self.current_time = time.localtime(time.time())
         self.reset_idle_time()
@@ -1818,6 +1856,17 @@ class Sakura:
                 self.passivemode = 1
             elif args[0:1] == ['leave']:
                 self.passivemode = 0
+        elif args[1:2] == ['collisionmode']:
+            if args[0:1] == ['enter']:
+                if args[2:3] == ['rect']:
+                    self.request_parent(
+                        'NOTIFY', 'set_collisionmode', True, rect=True)
+                else:
+                    self.request_parent(
+                        'NOTIFY', 'set_collisionmode', True)
+            elif args[0:1] == ['leave']:
+                self.request_parent(
+                    'NOTIFY', 'set_collisionmode', False)
         elif args[0:2] == ['set', 'alignmentondesktop'] and argc > 2:
             if args[2] == 'bottom':
                 if self.synchronized_session:
@@ -1848,6 +1897,20 @@ class Sakura:
                 self.surface.window_stayontop(True)
             elif args[2] == '!stayontop':
                 self.surface.window_stayontop(False)
+        elif args[0:2] == ['set', 'trayicon'] and argc > 2:
+            path = os.path.join(self.get_prefix(), os.fsencode(args[2]))
+            if os.path.exists(path):
+                self.status_icon.set_from_file(os.fsdecode(path)) # XXX
+            if argc > 3:
+                text = args[3]
+                if text:
+                    self.status_icon.set_has_tooltip(True)
+                    self.status_icon.set_tooltip_text(text)
+                else:
+                    self.status_icon.set_has_tooltip(False)
+            else:
+                self.status_icon.set_has_tooltip(False)
+            self.status_icon.set_visible(True)
         elif args[0:2] == ['set', 'wallpaper'] and argc > 2:
             path = os.path.join(self.get_prefix(), os.fsencode(args[2]))
             opt = None
@@ -2023,6 +2086,7 @@ class Sakura:
         r'\__t': __yen___t, 
         r'\v': __yen_v,
         r'\f': __yen_f,
+        r'\C': lambda *a: None, # dummy
         }
 
     def interpret_script(self):
