@@ -2,7 +2,7 @@
 #
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2012 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2013 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2003 by Shun-ichi TAHARA <jado@flowernet.gr.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -22,6 +22,8 @@ import logging
 
 from gi.repository import Gtk
 from gi.repository import Gdk
+from gi.repository import Pango
+from gi.repository import PangoCairo
 import cairo
 
 import ninix.seriko
@@ -168,7 +170,8 @@ class Surface:
             seriko[key] = ninix.seriko.get_actors(config)
         return seriko
 
-    def new(self, desc, alias, surface, name, prefix, tooltips):
+    def new(self, desc, alias, surface, name, prefix, tooltips,
+            default_sakura, default_kero):
         self.desc = desc
         self.__tooltips = tooltips
         self.name = name
@@ -215,9 +218,10 @@ class Surface:
                 composite_surface[key] = self.compose_elements(elements, config)
         surfaces.update(composite_surface)
         # check if necessary surfaces have been loaded
-        for key in ['0', '10']:
+        for key in [default_sakura, default_kero]:
             if key not in surfaces:
-                raise SystemExit('cannot load surface #{0} (abort)\n'.format(key))
+                raise SystemExit(
+                    'cannot load default surface #{0} (abort)\n'.format(key))
         self.__surfaces = surfaces
         # arrange surface configurations
         region = {}
@@ -294,8 +298,8 @@ class Surface:
             surface_window.destroy()
         self.window = []
         self.__surface = surface
-        self.add_window(0, '0', alias0, mayuna, bind['sakura'])
-        self.add_window(1, '10', alias1, mayuna, bind['kero'])
+        self.add_window(0, default_sakura, alias0, mayuna, bind['sakura'])
+        self.add_window(1, default_kero, alias1, mayuna, bind['kero'])
 
     def get_menu_pixmap(self):
         top_dir = self.prefix
@@ -906,6 +910,19 @@ class SurfaceWindow:
         scale = self.scale
         cr.scale(scale / 100.0, scale / 100.0)
         for part, x1, y1, x2, y2 in self.collisions:
+            if self.request_parent('GET', 'get_preference',
+                                   'check_collision_name'):
+                cr.set_operator(cairo.OPERATOR_SOURCE)
+                cr.set_source_rgba(0.4, 0.0, 0.0, 1.0) # XXX
+                cr.move_to(x1 + 2, y1)
+                font_desc = Pango.FontDescription()
+                font_desc.set_size(8 * Pango.SCALE)
+                layout = Pango.Layout(self.darea.get_pango_context())
+                layout.set_font_description(font_desc)
+                layout.set_wrap(Pango.WrapMode.CHAR) # XXX
+                layout.set_text(part, -1)
+                PangoCairo.update_layout(cr, layout)
+                PangoCairo.show_layout(cr, layout)
             cr.set_operator(cairo.OPERATOR_ATOP)
             cr.set_source_rgba(0.2, 0.0, 0.0, 0.4) # XXX
             cr.rectangle(x1, y1, x2 - x1, y2 - y1)
