@@ -2,7 +2,7 @@
 #
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2012 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2013 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2003-2005 by Shun-ichi TAHARA <jado@flowernet.gr.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -137,17 +137,26 @@ def main():
             os.makedirs(home_dir)
         except:
             raise SystemExit('Cannot create Home directory (abort)\n')
+    lockfile_path = os.path.join(ninix.home.get_ninix_home(), b'.lock')
+    if os.path.exists(lockfile_path):
+        with open(lockfile_path, 'r') as f:
+            abend = f.readline()
+        if not abend:
+            abend = None
+    else:
+        abend = None
     # aquire Inter Process Mutex (not Global Mutex)
-    with open(os.path.join(ninix.home.get_ninix_home(), b'.lock'), 'w') as f:
+    with open(lockfile_path, 'w') as f:
         try:
             ninix.lock.lockfile(f)
         except:
             raise SystemExit('ninix-aya is already running')
         # start
         logging.info('loading...')
-        app = Application(sstp_port)
+        app = Application(f, sstp_port)
         logging.info('done.')
-        app.run()
+        app.run(abend)
+        f.truncate(0)
         try:
             ninix.lock.unlockfile(f)
         except:
@@ -521,7 +530,9 @@ class Ghost(Holon):
 
 class Application:
 
-    def __init__(self, sstp_port=[9801, 11000]):
+    def __init__(self, lockfile, sstp_port=[9801, 11000]):
+        self.lockfile = lockfile
+        self.abend = None
         self.loaded = False
         self.confirmed = False
         self.console = Console(self)
@@ -585,6 +596,12 @@ class Application:
         if event_type == 'GET':
             return result
 
+    def set_collisionmode(self, flag, rect=False):
+        self.prefs.check_collision_button.set_active(bool(flag))
+        self.prefs.check_collision_name_button.set_active(not bool(rect))
+        self.prefs.update(commit=True) # XXX
+        self.notify_preference_changed()
+
     def do_install(self, filename):
         try:
             filetype, target_dir = self.installer.install(
@@ -831,7 +848,7 @@ class Application:
         ##for i, name in enumerate(self.get_ghost_names()):
         ##    logging.info(
         ##        'GHOST({0:d}): {1}'.format(i, name))
-        self.start_sakura(self.current_sakura, init=1)
+        self.start_sakura(self.current_sakura, init=1, abend=self.abend)
 
     def find_ghost_by_dir(self, directory):
         return directory if directory in self.ghosts else None
@@ -876,7 +893,8 @@ class Application:
     def exit_handler(self, sig_no, frame=None):
         self.close_all_ghosts(reason='shutdown')
 
-    def run(self):
+    def run(self, abend):
+        self.abend = abend
         if os.name == "nt":
             # The SIGTERM signal is not generated under Windows NT.
             import win32api
@@ -1011,7 +1029,8 @@ class Application:
         self.set_menu_sensitive(sakura.key, True)
         self.close_ghost(sakura)
 
-    def start_sakura(self, key, prev=None, vanished=0, init=0, temp=0):
+    def start_sakura(self, key, prev=None, vanished=0, init=0, temp=0,
+                     abend=None):
         sakura = self.ghosts[key].instance
         assert sakura is not None
         if prev is not None:
@@ -1037,9 +1056,15 @@ class Application:
             last_script = None
         sakura.notify_preference_changed()
         sakura.start(key, init, temp, vanished, ghost_changed,
-                     self_name, name, shell, last_script)
+                     self_name, name, shell, last_script, abend)
         self.set_menu_sensitive(key, False)
 
+    def update_working(self, ghost_name):
+        self.lockfile.truncate(0)
+        self.lockfile.seek(0)
+        self.lockfile.write(ghost_name)
+        self.lockfile.flush()        
+
     def notify_preference_changed(self):
         for sakura in self.get_working_ghost():
             sakura.notify_preference_changed()
